%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Purpose: Run trained PCG classifier and obtain classifier outputs
% Inputs:
% 1. header
% 2. recordings
% 3. trained model
%
% Outputs:
% 1. score
% 2. label
% 3. classes
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [score, label, classes] = team_testing_code(header, recordings, loaded_model)

% order classes the same as the NN
classes_murmur=loaded_model.model_murmur{1, 1}.Layers(20, 1).Classes';
classes_outcome=loaded_model.model_outcome{1, 1}.Layers(8, 1).Classes';

Fs = 4000; % 4000 Hz
timeStep = 5; % 5 seconds
N = timeStep * Fs;

MV = zeros(N,1);
TV = zeros(N,1);
PV = zeros(N,1);
AV = zeros(N,1);

current_header=header;
current_recordings1=recordings;

temp = strsplit(current_header{1});

% if sampling rate ~= 4000, resample
oldFs = str2double(temp{3});
if str2double(temp{3}) ~= Fs
    for k=1:length(current_recordings1)
        current_recordings2{k} = resample(current_recordings1{k}, Fs, oldFs);
    end
else
    current_recordings2 = current_recordings1;
end

for k=1:length(current_recordings2)
    if length(current_recordings2{k}) >= Fs*timeStep
        current_recordings3{k} = current_recordings2{k}(1:Fs*timeStep);
    else
        current_recordings3{k} = zeros(1,Fs*timeStep);
        current_recordings3{k}(1:length(current_recordings2{k})) = current_recordings2{k};
    end
end

for k=1:length(current_recordings3)
    current_recordings{k} = normalize(current_recordings3{k}, 'range', [-1 1]);
end

C = strsplit(current_header{1});
for m=2:str2double(cell2mat(C(2)))+1
    H = strsplit(current_header{m});
    H2 = cell2mat(H(1));
    if strcmp(H2,'MV')
        MV = current_recordings{m-1};
    elseif strcmp(H2,'TV')
        TV = current_recordings{m-1};
    elseif strcmp(H2,'PV')
        PV = current_recordings{m-1};
    elseif strcmp(H2,'AV')
        AV = current_recordings{m-1};
    end
end


murmurMV = predict(loaded_model.model_murmur{1},MV);
murmurTV = predict(loaded_model.model_murmur{2},TV);
murmurAV = predict(loaded_model.model_murmur{3},AV);
murmurPV = predict(loaded_model.model_murmur{4},PV);

outMV = predict(loaded_model.model_outcome{1},MV);
outTV = predict(loaded_model.model_outcome{2},TV);
outAV = predict(loaded_model.model_outcome{3},AV);
outPV = predict(loaded_model.model_outcome{4},PV);

score_murmur = (murmurMV + murmurTV + murmurAV + murmurPV)./4;
score_outcome = (outMV + outTV + outAV + outPV)./4;

[~, idxM] = max(score_murmur);
[~, idxO] = max(score_outcome);

label_murmur=zeros(1,length(classes_murmur));
label_murmur(idxM)=1;

label_outcome=zeros(1,length(classes_outcome));
label_outcome(idxO)=1;

classes=[classes_murmur classes_outcome];
score=[score_murmur score_outcome];
label=[label_murmur label_outcome];

end
