function model = team_training_code(input_directory,output_directory) % train_PCG_classifier
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Purpose: Train PCG classifiers and obtain the models
% Inputs:
% 1. input_directory
% 2. output_directory
%
% Outputs:
% 1. model: trained model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Reference: https://www.mathworks.com/help/deeplearning/ug/sequence-classification-using-1-d-convolutions.html

% Find text files
patient_files=dir(fullfile(input_directory,'*.txt'));
patient_files={patient_files.name};
%patient_files=sort(patient_files); % To help debugging
num_patient_files=length(patient_files);

fprintf('Loading data for %d patients...\n', num_patient_files)

% Extract classes from data
classes_murmur={};
classes_outcome={};

countPresent = 0;
countAbsent = 0;
countUnknown = 0;
absentList = [];
presentList = [];
unknownList = [];

for j=1:num_patient_files

    current_class_murmur=get_class_murmur(fullfile(input_directory,patient_files{j}));
    classes_murmur=unique([classes_murmur current_class_murmur]);
    if (strcmp(current_class_murmur, "Absent"))
        countAbsent = countAbsent + 1;
        absentList(countAbsent) = j;
    elseif (strcmp(current_class_murmur, "Present"))
        countPresent = countPresent + 1;
        presentList(countPresent) = j;
    elseif (strcmp(current_class_murmur, "Unknown"))
        countUnknown = countUnknown + 1;
        unknownList(countUnknown) = j;
    end

    current_class_outcome=get_class_outcome(fullfile(input_directory,patient_files{j}));
    classes_outcome=unique([classes_outcome current_class_outcome]);
end

biggestClass = max(countPresent, countAbsent);
m = 1;
k = j;
if (countPresent < biggestClass) 
    for k=j+1:j+biggestClass-countPresent
        patient_files{k} = patient_files{presentList(1+mod(m, countPresent))};
        m = m + 1;
    end
elseif (countAbsent < biggestClass)
    for k=j+1:j+biggestClass-countAbsent
        patient_files{k} = patient_files{absentList(1+mod(m, countAbsent))};
        m = m + 1;
    end
end

if (countUnknown < biggestClass)
    m = 1;
    for n=k+1:k+biggestClass-countUnknown
        patient_files{n} = patient_files{unknownList(1+mod(m, countUnknown))};
        m = m + 1;
    end
end

% reset this value after augmentation
num_patient_files=length(patient_files);

classes_murmur=sort(classes_murmur);
num_classes_murmur=length(classes_murmur);

classes_outcome=sort(classes_outcome);
num_classes_outcome=length(classes_outcome);

% Extracting features and labels
disp('Extracting features and labels...')

labels_murmur=categorical;
labels_outcome=categorical;

Fs = 4000; % 4000 Hz
timeStep = 5; % 5 seconds
N = timeStep * Fs;


MV = zeros(N,num_patient_files);
TV = zeros(N,num_patient_files);
PV = zeros(N,num_patient_files);
AV = zeros(N,num_patient_files);


for j=1:num_patient_files

    fprintf('%d/%d \n',j,num_patient_files)

    current_header=get_header(fullfile(input_directory,patient_files{j}));
    clear currentrecordings1 currentrecordings2 currentrecordings3 currentrecordings4 currentrecordings
    current_recordings1=load_recordings(input_directory,current_header);

    temp = strsplit(current_header{1});

    % if sampling rate ~= 4000, resample
    oldFs = str2double(temp{3});
    if str2double(temp{3}) ~= Fs
        for k=1:length(current_recordings1)
            current_recordings2{k} = resample(current_recordings1{k}, Fs, oldFs);
        end
    else
        current_recordings2 = current_recordings1;
    end

    for k=1:length(current_recordings2)
        if length(current_recordings2{k}) >= Fs*timeStep
            current_recordings3{k} = current_recordings2{k}(1:Fs*timeStep);
        else
            current_recordings3{k} = zeros(1,Fs*timeStep);
            current_recordings3{k}(1:length(current_recordings2{k})) = current_recordings2{k};
        end
    end

    for k=1:length(current_recordings3)
        current_recordings{k} = normalize(current_recordings3{k}, 'range', [-1 1]);
    end

    C = strsplit(current_header{1});
    for m=2:str2double(cell2mat(C(2)))+1
        H = strsplit(current_header{m});
        H2 = cell2mat(H(1));
        if strcmp(H2,'MV')
            MV(:,j) = current_recordings{m-1};
        elseif strcmp(H2,'TV')
            TV(:,j) = current_recordings{m-1};
        elseif strcmp(H2,'PV')
            PV(:,j) = current_recordings{m-1};
        elseif strcmp(H2,'AV')
            AV(:,j) = current_recordings{m-1};
        end
    end

    labels_murmur(j)=get_class_murmur(fullfile(input_directory,patient_files{j}));
    labels_outcome(j)=get_class_outcome(fullfile(input_directory,patient_files{j}));

end

labels_murmur = labels_murmur';
labels_outcome = labels_outcome';

%% train RF

disp('Training the model...')

MV = num2cell(MV,1);
TV = num2cell(TV,1);
PV = num2cell(PV,1);
AV = num2cell(AV,1);

MV = MV';
TV = TV';
PV = PV';
AV = AV';

filterSize = 3;
numFilters = 24;
numFeaturesTemp1 = max(size(MV{1},1), size(TV{1},1));
numFeaturesTemp2 = max(size(PV{1},1), size(AV{1},1));
numFeatures = max(numFeaturesTemp1, numFeaturesTemp2);

layers = [ ...
    sequenceInputLayer(numFeatures)
    convolution1dLayer(filterSize,numFilters,Padding="causal")
    reluLayer
    batchNormalizationLayer
    convolution1dLayer(filterSize,2*numFilters,Padding="causal")
    reluLayer
    batchNormalizationLayer
    convolution1dLayer(filterSize,4*numFilters,Padding="causal")
    reluLayer
    batchNormalizationLayer
    convolution1dLayer(filterSize,8*numFilters,Padding="causal")
    reluLayer
    batchNormalizationLayer
    convolution1dLayer(filterSize,16*numFilters,Padding="causal")
    reluLayer
    batchNormalizationLayer
    globalAveragePooling1dLayer
    fullyConnectedLayer(num_classes_murmur)
    softmaxLayer
    classificationLayer];

layers_outcome = [ ...
    sequenceInputLayer(numFeatures)
    convolution1dLayer(filterSize,numFilters,Padding="causal")
    reluLayer
    batchNormalizationLayer
    globalAveragePooling1dLayer
    fullyConnectedLayer(num_classes_outcome)
    softmaxLayer
    classificationLayer];


miniBatchSize = 32;

options = trainingOptions("adam", ...
    MiniBatchSize=miniBatchSize, ...
    MaxEpochs=60, ...
    SequencePaddingDirection="left", ... 
    Shuffle="every-epoch", ...
    Verbose=0);

options_outcome = trainingOptions("adam", ...
    MiniBatchSize=miniBatchSize, ...
    MaxEpochs=15, ...
    SequencePaddingDirection="left", ... 
    Verbose=0);

classificationSVM_MV = trainNetwork(MV, labels_murmur,layers,options);
classificationSVM_TV = trainNetwork(TV, labels_murmur,layers,options);
classificationSVM_AV = trainNetwork(AV, labels_murmur,layers,options);
classificationSVM_PV = trainNetwork(PV, labels_murmur,layers,options);

outcome_MV = trainNetwork(MV, labels_outcome,layers_outcome,options_outcome);
outcome_TV = trainNetwork(TV, labels_outcome,layers_outcome,options_outcome);
outcome_AV = trainNetwork(AV, labels_outcome,layers_outcome,options_outcome);
outcome_PV = trainNetwork(PV, labels_outcome,layers_outcome,options_outcome);

model_murmur = {classificationSVM_MV classificationSVM_TV classificationSVM_AV classificationSVM_PV};
model_outcome = {outcome_MV outcome_TV outcome_AV outcome_PV};
save_model(model_murmur,classes_murmur,model_outcome,classes_outcome,output_directory);

disp('Done.')

end

function save_model(model_murmur,classes_murmur,model_outcome,classes_outcome,output_directory) %save_PCG_model
% Save results.
filename = fullfile(output_directory,'model.mat');
save(filename,'model_murmur','classes_murmur','model_outcome','classes_outcome','-v7.3');

disp('Done.')
end

function class=get_class_murmur(input_header)

current_header=get_header(input_header);

class=current_header(startsWith(current_header,'#Murmur'));
class=strsplit(class{1},':');
class=strtrim(class{2});

end

function class=get_class_outcome(input_header)

current_header=get_header(input_header);

class=current_header(startsWith(current_header,'#Outcome'));
class=strsplit(class{1},':');
class=strtrim(class{2});

end

function current_header=get_header(input_header)

current_header=fileread(input_header);
current_header=strsplit(current_header,'\n');

end

function current_recordings=load_recordings(input_directory,current_header)

recording_files=get_recording_files(current_header);

current_recordings={};

for j=1:length(recording_files)

    current_recordings{j}=audioread(fullfile(input_directory,strtrim(recording_files{j})));

end

end

function recording_files=get_recording_files(current_header)

recording_files={};

num_locations=strsplit(current_header{1},' ');
num_locations=str2double(num_locations{2});

for j=2:num_locations+1

    current_line=strsplit(current_header{j},' ');
    recording_files{j-1}=current_line{3};

end

end
